unit SiField;

{ SIConvertField:
    TField that converts the data shown (lb, oz..) and stored (kg).
  Version 0.1 May-25-1997  (C) 1997 Christoph R. Kirchner
  !! This component is currently UNDER CONSTRUCTION !!
}
{ Users of this unit must accept this disclaimer of  warranty:
    "This unit is supplied as is. The author disclaims all warranties,
    expressed or implied, including, without limitation, the warranties
    of merchantability and of fitness for any purpose.
    The author assumes no liability for damages, direct or
    consequential, which may result from the use of this unit."

  This Unit is donated to the public as public domain.

  This Unit can be freely used and distributed in commercial and
  private environments provided this notice is not modified in any way.

  If you do find this Unit handy and you feel guilty for using such a
  great product without paying someone - sorry :-)

  Please forward any comments or suggestions to Christoph Kirchner at:
  ckirchner@geocities.com

  Maybe you can find an update of this component at my
  "Delphi Component Building Site":
  http://www.geocities.com/SiliconValley/Heights/7874/delphi.htm
}


interface

uses SysUtils, Classes, DB, DBTables, DsgnIntf, CalcFld;

type

  TNonSIUnit =
   (nsiuUnknown,
  { mass: SI-base is kg: }
    nsiuPoundLB,    { lb }
    nsiuPound2KG,   { as used e.g. in Germany: 2 pound = 1 kg }
    nsiuOunce,      { oz }
    nsiuStone,      { st }
    nsiuQuarter,
    nsiuCwt,        { GB cwt }
    nsiuShortCwt,   { US cwt }
    nsiuTon1000KG,  { metric: 1 t = 1000 kg }
    nsiuTonGB,      { 20 GB cwt }
    nsiuShortTon,   { US t }
  { lenght: SI-base is m: }
  { surface: SI-base is m^2: }
  { capacity: SI-base is m^3:
    (NOT litre, set Multiply to 0.001 if you store litres) }
    nsiuGallonGB,   { GB gal }
    nsiuGallonUS    { US gal }
   );

const
  SIConvertFactor: array[TNonSIUnit] of Double =
    (1.0,
  { mass: SI-base is kg: }
     0.453592,      { lb }
     0.5,           { pound as used e.g. in Germany: 2 pound = 1 kg }
     0.02835,       { oz }
     6.35,          { st }
     12.701,        { Quarter }
     50.802,        { GB cwt }
     45.359,        { US cwt }
     1000.0,        { metric: 1 t = 1000 kg }
     1016.05,       { t: 20 GB cwt }
     907.185,       { US t }
  { lenght: SI-base is m: }
  { surface: SI-base is m^2: }
  { capacity: SI-base is m^3:
    (NOT litre, set Multiply to 0.001 if you store litres) }
     0.004546,      { GB gal }
     0.003785       { US gal }
    );

type

  TCustomSIConvertField = class(TCustomAutoCalcFloatField)
  private
    FSIField: string;
    FNonSIUnit: TNonSIUnit;
    FFactor: Double;
    FMultiply: Double;
    procedure SetNonSIUnit(Value: TNonSIUnit);
    procedure SetFactor(Value: Double);
    procedure SetMultiply(Value: Double);
  protected
    procedure ConvertToBase(var Value: Double); override;
    procedure ConvertFromBase(var Value: Double); override;
  public
    constructor Create(AOwner: TComponent); override;
    property NonSIUnit: TNonSIUnit
      read FNonSIUnit write SetNonSIUnit default nsiuUnknown;
    property Factor: Double read FFactor write SetFactor;
    property Multiply: Double read FMultiply write SetMultiply;
  end;

  TSIConvertField = class(TCustomSIConvertField)
  published
    property NonSIUnit;
    property Factor;
    property Multiply;
  end;

  { sample of a special-purpose SIConvertField:
  TPoundFloatField = class(TCustomSIConvertField)
  protected
    constructor Create(AOwner: TComponent); override;
  end;
  }

procedure Register;

implementation

{ TCustomSIConvertField -------------------------------------------------- }

constructor TCustomSIConvertField.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFactor := 1.0;
  FMultiply := 1.0;
  FNonSIUnit := nsiuUnknown;
end;

procedure TCustomSIConvertField.SetNonSIUnit(Value: TNonSIUnit);
begin
  if (FNonSIUnit <> Value) then
  begin
    FNonSIUnit := Value;
    if (FNonSIUnit <> nsiuUnknown) then
      FFactor := SIConvertFactor[FNonSIUnit];
    if not (csLoading in ComponentState) and
       not (csDesigning in ComponentState) then
      DataChanged;
  end;
end;

procedure TCustomSIConvertField.SetFactor(Value: Double);
begin
  if (FFactor <> Value) then
  begin
    FFactor := Value;
    FNonSIUnit := nsiuUnknown;
    if not (csLoading in ComponentState) and
       not (csDesigning in ComponentState) then
      DataChanged;
  end;
end;

procedure TCustomSIConvertField.SetMultiply(Value: Double);
begin
  if (FMultiply <> Value) then
  begin
    FMultiply := Value;
    if not (csLoading in ComponentState) and
       not (csDesigning in ComponentState) then
      DataChanged;
  end;
end;

procedure TCustomSIConvertField.ConvertToBase(var Value: Double);
begin
    Value := Value * FFactor * FMultiply;
end;

procedure TCustomSIConvertField.ConvertFromBase(var Value: Double);
begin
  if (FFactor <> 0.0) and (FMultiply <> 0.0) then
    Value := (Value / FFactor) / FMultiply;
end;


{ TPoundFloatField ---------------------------------------------------------

constructor TPoundFloatField.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  NonSIUnit := nsiuPoundLB;
end;


{ registration ------------------------------------------------------------}

procedure Register;
begin
  RegisterFields([TSIConvertField]);
{ RegisterFields([TPoundFloatField]); }
  CalcFld.Register; { register BaseFieldProperty }
end;

end.
