unit CalcFld;

{ CustomAutoCalcFloatField:

  TField that converts the data shown and stored.

  Version 0.11 Oct-30-1997  (C) 1997 Christoph R. Kirchner
  !! This component is currently UNDER CONSTRUCTION !!
}
{ Users of this unit must accept this disclaimer of warranty:
    "This unit is supplied as is. The author disclaims all
    warranties, expressed or implied, including, without
    limitation, the warranties of merchantability and of fitness
    for any purpose.
    The author assumes no liability for damages, direct or
    consequential, which may result from the use of this unit."

  This Unit is donated to the public as public domain.

  This Unit can be freely used and distributed in commercial and
  private environments provided this notice is not modified in any way.

  If you do find this Unit handy and you feel guilty for using such a
  great product without paying someone - sorry :-)

  Please forward any comments or suggestions to Christoph Kirchner at:
  ckirchner@geocities.com

  Maybe you can find an update of this component at my
  "Delphi Component Building Site":
  http://www.geocities.com/SiliconValley/Heights/7874/delphi.htm
}


interface

uses SysUtils, Classes, DB, DBTables, DsgnIntf;

type

  TCustomAutoCalcFloatField = class(TFloatField)
  private
    FDataSource: TDataSource;
    FBaseField: string;
    procedure DataChange(Sender: TObject; Field: TField);
  protected
  {$IFDEF WIN32}
    function GetAsVariant: Variant; override;
  {$ENDIF DEF WIN32}
    function DoGetData(Buffer: Pointer): Boolean; virtual;
    function GetAsFloat: Double; override;
    function GetAsString: string; override;
    procedure GetText(var Text: string; DisplayText: Boolean); override;
    procedure SetAsFloat(Value: Double); override;
    procedure ConvertToBase(var Value: Double); virtual; abstract;
    procedure ConvertFromBase(var Value: Double); virtual; abstract;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property BaseField: string read FBaseField write FBaseField;
  end;

procedure Register;

implementation

type
  TPublicDataEventDataSet = class(TDataSet)
  public
  { dummy to make DataEvent public: (never gets called) }
    procedure DataEvent(Event: TDataEvent; Info: Longint); override;
    {$IFDEF WIN32} abstract; {$ENDIF DEF WIN32}
  end;

{$IFNDEF WIN32}
procedure TPublicDataEventDataSet.DataEvent(Event: TDataEvent; Info: Longint);
begin { never gets called } end;
{$ENDIF NDEF WIN32}


{ TCustomAutoCalcFloatField -------------------------------------------------- }

constructor TCustomAutoCalcFloatField.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FBaseField := '';
  FDataSource := nil;
end;

destructor TCustomAutoCalcFloatField.Destroy;
begin
  if Assigned(FDataSource) then
    FDataSource.Free;
  inherited Destroy;
end;


type
{ This is needed to get write-access to FFieldNo: }
  TPublicField = class(TComponent)
  public
{$IFDEF WIN32}
  {$IFDEF VER100} { Delphi 3.0: }
    FDataSet: TDataSet;
    FFieldName: string;
    FDataType: TFieldType;
    FReadOnly: Boolean;
    FFieldKind: TFieldKind;
    FAlignment: TAlignment;
    FVisible: Boolean;
    FRequired: Boolean;
    FValidating: Boolean;
    FSize: Word;
    FOffset: Word;
    FFieldNo: Integer;
  {$ELSE DEF VER100} { Delphi 2.0: }
    FDataSet: TDataSet;
    FFieldName: string;
    FDataType: TFieldType;
    FReadOnly: Boolean;
    FAlignment: TAlignment;
    FVisible: Boolean;
    FRequired: Boolean;
    FValidating: Boolean;
    FSize: Word;
    FDataSize: Word;
    FFieldNo: Integer;
  {$ENDIF DEF VER100}
  end;
{$ELSE DEF WIN32}
    FDataSet: TDataSet;
    FFieldName: PString;
    FDataType: TFieldType;
    FReadOnly: Boolean;
    FCalculated: Boolean;
    FAlignment: TAlignment;
    FVisible: Boolean;
    FRequired: Boolean;
    FValidating: Boolean;
    FReserved: Byte;
    FSize: Word;
    FDataSize: Word;
    FFieldNo: Integer;
  end;
{$ENDIF DEF WIN32}

function TCustomAutoCalcFloatField.DoGetData(Buffer: Pointer): Boolean;
begin
  if Calculated and not (csDesigning in ComponentState) then
  begin
    if (FieldNo <= 0) and (DataSet <> nil) and (FBaseField <> '') and
       (DataSet.FieldByName(FBaseField) <> nil) then
    begin
    { Use the same FieldNo as the SI-field: }
      TPublicField(self).FFieldNo := DataSet.FieldByName(FBaseField).FieldNo;
    { Avoid that the DataSet nags about a non-existing FieldName: }
      DataSet.FieldDefs.Add(FieldName, DataType, Size, Required);
      if (FDataSource = nil) then
      begin
        FDataSource := TDataSource.Create(Self);
        FDataSource.DataSet := DataSet;
        FDataSource.OnDataChange := DataChange;
      end;
    end;
  end;
  result := GetData(Buffer);
end;

procedure TCustomAutoCalcFloatField.DataChange(Sender: TObject; Field: TField);
begin
  if (Field <> nil) and (FieldNo >= 0) and
     (DataSet <> nil) and (BaseField <> '') and
     (DataSet.FieldByName(BaseField) = Field) then
    with TPublicDataEventDataSet(DataSet) do
      if {$IFDEF WIN32} (State <> dsFilter) and {$ENDIF DEF WIN32}
         (State <> dsCalcFields) then
        DataEvent(deRecordChange, 0);
{   DataChanged; }
end;

function TCustomAutoCalcFloatField.GetAsFloat: Double;
begin
  if DoGetData(@Result) then
    ConvertFromBase(Result)
  else
    Result := 0;
end;

function TCustomAutoCalcFloatField.GetAsString: string;
var
  F: Double;
begin
  if DoGetData(@F) then
  begin
    ConvertFromBase(F);
    Result := FloatToStr(F);
  end
  else
    Result := '';
end;

{$IFDEF WIN32}
function TCustomAutoCalcFloatField.GetAsVariant: Variant;
var
  F: Double;
begin
  if DoGetData(@F) then
  begin
    ConvertFromBase(F);
    Result := F;
  end
  else
    Result := Null;
end;
{$ENDIF DEF WIN32}

procedure TCustomAutoCalcFloatField.GetText(var Text: string; DisplayText: Boolean);
var
  Format: TFloatFormat;
  Digits: Integer;
  FmtStr: string;
  F: Double;
begin
  if DoGetData(@F) then
  begin
    ConvertFromBase(F);
    if DisplayText or (EditFormat = '') then
      FmtStr := DisplayFormat
    else
      FmtStr := EditFormat;
    if FmtStr = '' then
    begin
      if Currency then
      begin
        if DisplayText then
          Format := ffCurrency
        else
          Format := ffFixed;
        Digits := CurrencyDecimals;
      end
      else
      begin
        Format := ffGeneral;
        Digits := 0;
      end;
      Text := FloatToStrF(F, Format, Precision, Digits);
    end
    else
      Text := FormatFloat(FmtStr, F);
  end
  else
    Text := '';
end;

procedure TCustomAutoCalcFloatField.SetAsFloat(Value: Double);
begin
  ConvertToBase(Value);
  inherited SetAsFloat(Value);
  if Calculated and (FieldNo >= 0) and (DataSet <> nil) and
     (BaseField <> '') then
    with TPublicDataEventDataSet(DataSet) do
      if (State <> dsCalcFields) and
        {$IFDEF WIN32}
         (State <> dsFilter) and
        {$ENDIF DEF WIN32}
         (FieldByName(BaseField) <> nil) then
        DataEvent(deRecordChange, 0);
{       DataEvent(deFieldChange, Longint(FieldByName(BaseField))); {}
end;


{ Property-Editors and registration -----------------------------------------}

type

  TBaseFieldProperty = class(TStringProperty)
    procedure GetValueList(List: TStrings);
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

function TBaseFieldProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paMultiSelect];
end;

procedure TBaseFieldProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
  Values: TStringList;
begin
  Values := TStringList.Create;
  try
    GetValueList(Values);
    for I := 0 to Values.Count - 1 do Proc(Values[I]);
  finally
    Values.Free;
  end;
end;

procedure TBaseFieldProperty.GetValueList(List: TStrings);
var
  I: Integer;
  ADataSet: TDataSet;
  AAutoCalcFloatField: TCustomAutoCalcFloatField;
begin
  AAutoCalcFloatField := TCustomAutoCalcFloatField(GetComponent(0));
  ADataSet := AAutoCalcFloatField.DataSet;
  if (ADataSet <> nil) then
    with ADataSet do
    begin
      for i := 0 to FieldCount - 1 do
        with Fields[i] do
          if (DataType = ftFloat) and
             (FieldName <> AAutoCalcFloatField.FieldName) and
             not Calculated then
            List.Add(FieldName);
    end;
end;

(* Property to keep Calculated readonly at designtime:
type

  TSICalculatedProperty = class(TEnumProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure SetValue(const Value: string); override;
  end;

function TSICalculatedProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paReadOnly];
end;

procedure TSICalculatedProperty.SetValue(const Value: string);
begin { ignore } end;
*)

procedure Register;
begin
  RegisterPropertyEditor(TypeInfo(string), TCustomAutoCalcFloatField,
                         'BaseField',
                         TBaseFieldProperty);
{ RegisterPropertyEditor(TypeInfo(boolean), TCustomAutoCalcFloatField,
                         'Calculated',
                         TSICalculatedProperty); }
end;

end.
